/*
 * Copyright 2004-2012 ICEsoft Technologies Canada Corp.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS
 * IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package org.icefaces.mobi.component.list;


import org.icefaces.mobi.utils.HTML;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;
import java.io.IOException;
import java.util.logging.Logger;


public class OutputListItemRenderer extends Renderer {
    private static Logger logger = Logger.getLogger(OutputListItemRenderer.class.getName());

    public void encodeBegin(FacesContext facesContext, UIComponent uiComponent)
            throws IOException {
        ResponseWriter writer = facesContext.getResponseWriter();
        String clientId = uiComponent.getClientId(facesContext);
        OutputListItem item = (OutputListItem) uiComponent;

        writer.startElement(HTML.LI_ELEM, uiComponent);
        writer.writeAttribute(HTML.ID_ATTR, clientId, HTML.ID_ATTR);
        String userDefinedClass = item.getStyleClass();
        String styleClass = OutputListItem.OUTPUTLISTITEM_CLASS;
        if (item.isGroup()) {
            styleClass = OutputListItem.OUTPUTLISTITEMGROUP_CLASS;
            if (userDefinedClass != null) {
                styleClass += " " + userDefinedClass;
            }
        } else {
            if (userDefinedClass != null) {
                styleClass = OutputListItem.OUTPUTLISTITEM_CLASS + " " + userDefinedClass;
            }
        }
        writer.writeAttribute("class", styleClass, "styleClass");
        writer.startElement(HTML.DIV_ELEM, uiComponent);
        writer.writeAttribute("class", OutputListItem.OUTPUTLISTITEMDEFAULT_CLASS, null);
    }

    public void encodeEnd(FacesContext facesContext, UIComponent uiComponent)
            throws IOException {
        ResponseWriter writer = facesContext.getResponseWriter();
        writer.endElement(HTML.DIV_ELEM);
        writer.endElement(HTML.LI_ELEM);
    }

}
